#include "configuration.h"
#include "extruder.h"
#include "temperature.h"

extruder::extruder()
{
/* Trying to have only one 5V line on the mcu adc, reasoning being just good practice & theory, although for me seems to have none or little effect. See http://www.arduino.cc/en/Reference/AnalogReference */
  #if AREF_VOLTS == FIVE
  analogReference(EXTERNAL); /* if Aref is hardwired to 5v (it is so in the typical EC22 board, the connection is right-side-above A6 pin, just does not appear in schematic) */
  #endif
  #if AREF_VOLTS == ZERO
  analogReference(DEFAULT);  /* if Aref is disconnected from 5v (EC22 if you are hard cutting off the pcb trace linking Aref to 5v) - this line itself is redundacy coz internal mcu 5v is still default one */
  #endif
  pinMode(BED_OUTPUT, OUTPUT);
  pinMode(FAN_OUTPUT, OUTPUT);
  pinMode(HEATER_OUTPUT, OUTPUT);
#ifdef E_MAX6675_THERMOCOUPLE
  pinMode(SO, INPUT);
  pinMode(SCK, OUTPUT);
  pinMode(TC_0, OUTPUT); 
  digitalWrite(TC_0,HIGH);  // Disable MAX6675
#endif
#ifdef E_AD595_THERMOCOUPLE || E_THERMISTOR
  pinMode(E_TEMP_PIN, INPUT);
#endif
#ifdef B_AD595_THERMOCOUPLE || B_THERMISTOR
  pinMode(B_TEMP_PIN, INPUT);
#endif 
 /* Starting with these */  
 targetTemperature = 0;  currentTemperature = 0;   manageCount = 0; 
 targetBedTemperature = 0; currentBedTemperature = 0; manageBedCount = ( SLOW_CLOCK / 2 ); /* Bed starting SLOW_CLOCK/2 -> to antiphase analog reads for less adc jitter, theory, coz for me seems to have none or little effect*/
 skipCount = 0; /* skipcount is for max6675 which is only for E */
}


// E_Probe  *************************************** E Probe Readin'
int extruder::internalTemperature() 
{ 
    #ifdef E_AD595_THERMOCOUPLE /* AD595 outputs 10mV/C: 1V=100C; 2V=200C, etc, thus needing reference voltage higher than 1.1V option */
      analogRead(E_TEMP_PIN); analogRead(E_TEMP_PIN); analogRead(E_TEMP_PIN); /* the 3rd or 4th analog reading should be jitter clean */
      return analogRead(E_TEMP_PIN) * 500 / 1024; 
//      int raw = 0; /* read nr of samples */ for (byte i=0; i<TEMPERATURE_SAMPLES; i++) raw += analogRead(E_TEMP_PIN); /*make average and use it*/ raw = ((raw/TEMPERATURE_SAMPLES) * 500.0) / 1024.0; /* send back value */ return raw; 
    #endif  

    #ifdef E_MAX6675_THERMOCOUPLE 
      skipCount++;   if (skipCount>SKIP_CLOCK_COUNT)  {    skipCount=0;  }       else    { 
      int value = 0;       byte error_tc;      digitalWrite(TC_0, 0); /* Enable device */
      digitalWrite(SCK,1);      digitalWrite(SCK,0);       /* Cycle the clock for dummy bit 15 */ 
      /* Read bits 14-3 from MAX6675 for the Temp Loop for each bit reading the value */
      for (int i=11; i>=0; i--) { digitalWrite(SCK,1);  /* Set Clock to HIGH */  value += digitalRead(SO) << i;  /* Read data and add it to our variable */   digitalWrite(SCK,0);  /* Set Clock to LOW */  }
      /* Read the TC Input inp to check for TC Errors */
                                  digitalWrite(SCK,1); /* Set Clock to HIGH */   error_tc = digitalRead(SO);     /* Read data */                              digitalWrite(SCK,0);  /* Set Clock to LOW */
      digitalWrite(TC_0, 1); /* Disable Device */ if(error_tc) currentTemperature = 2000; else currentTemperature = value/4;  }
    #endif

    #ifdef E_THERMISTOR
      int raw = analogRead(E_TEMP_PIN);      int celsius = raw;      byte i;
      for (i=1; i<NUMTEMPS; i++) { if (temptable[i][0] > raw)   { celsius = temptable[i-1][1] + (raw - temptable[i-1][0]) * (temptable[i][1] - temptable[i-1][1]) / (temptable[i][0] - temptable[i-1][0]); break;  }  }
      /* Overflow: Set to last value in the table */      if (i == NUMTEMPS) celsius = temptable[i-1][1]; 
//      /* Clamp to byte */  if (celsius > 500) celsius = 500; else if (celsius < 0) celsius = 0; 
      else return celsius; 
    #endif  
}


// B_Probe  *************************************** B Probe Readin'
int extruder::internalBedTemperature() 
{ 
    #ifdef B_AD595_THERMOCOUPLE /* AD595 outputs 10mV/C: 1V=100C; 2V=200C, etc, thus needing reference voltage higher than 1.1V option */
      analogRead(B_TEMP_PIN); analogRead(B_TEMP_PIN); analogRead(B_TEMP_PIN); /* the 3rd or 4th analog reading should be jitter clean */
      return analogRead(B_TEMP_PIN) * 500 / 1024; 
//      int raw = 0; /* read nr of samples */ for (byte i=0; i<TEMPERATURE_SAMPLES; i++) raw += analogRead(B_TEMP_PIN); /*make average*/ raw = ((raw/TEMPERATURE_SAMPLES) * 500.0) / 1024.0; /* send back value */ return raw; 
    #endif  

    #ifdef B_THERMISTOR
      int raw = analogRead(B_TEMP_PIN);      int celsius = raw;      byte i;
      for (i=1; i<NUMTEMPS; i++) { if (bedtemptable[i][0] > raw)   { celsius = bedtemptable[i-1][1] + (raw - bedtemptable[i-1][0]) * (bedtemptable[i][1] - bedtemptable[i-1][1]) / (bedtemptable[i][0] - bedtemptable[i-1][0]); break;  }  }
      /* Overflow: Set to last value in the table */      if (i == NUMTEMPS) celsius = bedtemptable[i-1][1];
//      /* Clamp to byte */       if (celsius > 500) celsius = 500; else if (celsius < 0) celsius = 0; 
      else return celsius;
    #endif  
}


// E_temp bang-bang control
//void extruder::controlTemperature() {currentTemperature = internalTemperature(); if(targetTemperature > currentTemperature) digitalWrite(HEATER_OUTPUT, HIGH); else digitalWrite(HEATER_OUTPUT, LOW); }
void extruder::controlTemperature() {currentTemperature = internalTemperature(); if(currentTemperature > TEMPSHUT_MAX || currentTemperature < TEMPSHUT_MIN) shutdown(); else
                                                                                 if(targetTemperature > currentTemperature) digitalWrite(HEATER_OUTPUT, HIGH); else digitalWrite(HEATER_OUTPUT, LOW); }

// B_temp bang-bang control
//void extruder::controlBedTemperature() {currentBedTemperature = internalBedTemperature(); if(targetBedTemperature > currentBedTemperature) digitalWrite(BED_OUTPUT, HIGH); else digitalWrite(BED_OUTPUT, LOW); }
void extruder::controlBedTemperature() {currentBedTemperature = internalBedTemperature(); if(currentBedTemperature > TEMPSHUT_MAX || currentBedTemperature < TEMPSHUT_MIN) shutdown(); else
                                                                                          if(targetBedTemperature > currentBedTemperature) digitalWrite(BED_OUTPUT, HIGH); else digitalWrite(BED_OUTPUT, LOW); }


// common stuff
void extruder::setCooler(byte e_speed) {  analogWrite(FAN_OUTPUT, e_speed);  }
void extruder::shutdown() {  /* Heater off;*/  setTemperature(0);  setBedTemperature(0); digitalWrite(HEATER_OUTPUT, LOW); digitalWrite(BED_OUTPUT, LOW);  }  
void extruder::manage() { manageCount++; manageBedCount++; if(manageCount > SLOW_CLOCK) slowManage(); if(manageBedCount > SLOW_CLOCK) slowBedManage();  }
// E_ stuff
void extruder::slowManage() {  manageCount = 0;  /* blink(true); */  controlTemperature();   }
void extruder::waitForTemperature() {}
void extruder::setTemperature(int et) {  targetTemperature = et;  }
int extruder::getTemperature() {  currentTemperature = internalTemperature();  return currentTemperature;  }  /* E merge fara current = internal, dar bed nu - de ce? */
// B_ stuff
void extruder::slowBedManage() {  manageBedCount = 0;  /* blink(true); */  controlBedTemperature();   }
void extruder::waitForBedTemperature() {}
void extruder::setBedTemperature(int bt) {  targetBedTemperature = bt;  }
int extruder::getBedTemperature() {  currentBedTemperature = internalBedTemperature();  return currentBedTemperature;  }
/* ==================== see hostcom.h [setETemp etemp = et and setBTemp btemp = bt] why not name it like in hostcom but ofc if using slave mcu then benefit = zero; just in single mcu it could make sense */

// abuse comms
char* extruder::processCommand(char command[])
{ reply[0] = 0;   switch(command[0])
    {   
      case WAIT_T: { waitForTemperature(); waitForBedTemperature(); break;  }
      case COOL:    setCooler(atoi(&command[1]));    break;
      case PING:    break;
      case SHUT:    shutdown();    break;  
      case SET_T:    setTemperature(atoi(&command[1]));    break;
      case GET_T:    itoa(getTemperature(), reply, 10);    break;
      case SET_BED_T:    setBedTemperature(atoi(&command[1]));    break;
      case GET_BED_T:    itoa(getBedTemperature(), reply, 10);    break;
      default:    return 0; /* Flag up dud command */  
    }
 return reply; 
}



//void extruder::waitForTemperature(){  byte seconds = 0;  bool warming = true;  waitcount = 0;  newT = 0;  oldT = newT;  
//    while (true){ newT += getTemperature(); waitcount++; if(waitcount > 5) 
//    { newT = newT/5; if(newT >= targetTemperature - HALF_DEAD_ZONE) 
//      { warming = false; if(seconds > WAIT_AT_TEMPERATURE) return; else seconds++; } if(warming) { if(newT > oldT) oldT = newT; else { /* Temp isn't increasing - hardware error */  temperatureError();  return; } } 
//    newT = 0; waitcount = 0; } for(int i = 0; i < 1000; i++)    { manage(); delay(1);}  } }
//
//void extruder::temperatureError(){/* Heater off;*/  setTemperature(0);  setBedTemperature(0); } /* This is a fatal error - something is wrong with the heater. Cant figure a path to report to host thorugh master */


